// This program demonstrates an exception handling
// problem with Borland C++ 5.02 and 5.2.  When this
// particular program is run, a single object of type
// Derived is created in routine Test().  This automatic
// object should be destroyed when Test() is exited via
// an exception, which is exactly what happens.
// Unfortunately, the destructor for the object is called
// twice instead of just once, which will cause trouble
// in any class that allocates and frees resources.
//
// The key to the problem seems to be the existence of
// a derived class with no explicitly defined destructor.
//
// To see this program in action, compile with the
// following command line:
//
//  bcc32 Jul98.cpp
//

#include <iostream.h>

class Base
{
public:
    Base( char *name ) : m_Name( name ) {
        cout << "Calling Base ctor for object "
             << name << "\n";
        m_Valid = true;
    }
    virtual ~Base() {
        cout << "Calling Base dtor for object "
             << m_Name << "\n";
        cout << "Object is "
             << ( m_Valid ? "valid\n" : "invalid!\n" );
        m_Valid = false;
    }
protected:
    bool m_Valid;
    char *m_Name;
};

class Derived : public Base
{
public:
    Derived( char *name ) : Base( name ) {
        cout << "Calling Derived ctor for object "
             << name << "\n";
    };
};

void ThrowException()
{
    throw 0;
}

int Test()
{
    Derived foo( "foo" );
    try {
        ThrowException();
    }
    catch (...)
    {
        return 0; //Step d: return inside catch block
    }
    return 1;
};

int main( void )
{
    Test();
    return 1;
};





